#include "liblist.h"

// methods (functions) on list_t
void list_init(list_t* l) {
    // empty list
    l->head = NULL;
}

// insert method: put something into FRONT of list
void list_insert(list_t* l, int key, void* data) {
    node_t* n = malloc(sizeof(node_t));
    assert(n != NULL);
    n->key = key;
    n->data = data;
    n->next = l->head;
    l->head = n;
}

// insert method #2: insert into BACK of list
void list_insert_back(list_t* l, int key) {
    node_t* n = malloc(sizeof(node_t));
    assert(n != NULL);
    n->key = key;
    n->data = data;
    n->next = NULL;

    // special case: list is empty
    if (l->head == NULL) {
	l->head = n;
	return;
    }

    // general case: non-empty, have to find end
    node_t* curr;
    node_t* prev;
    curr = prev = l->head;
    while (curr) {
	prev = curr;
	curr = curr->next;
    }
    prev->next = n;
}

// 1 if present, 0 otherwise
int list_present(list_t* l, int key) {
    node_t* curr = l->head;
    while (curr && curr->key != key)
	curr = curr->next;
    return curr != NULL;
}

void list_delete(list_t* l, int key) {
    if (l->head == NULL) {	
	printf("empty list!\n");
	return;
    }

    if (l->head->key == key) {
	printf("delete from head\n");
	node_t* tmp = l->head;
	l->head = l->head->next;
	free(tmp);
	return;
    }

    node_t* curr;
    node_t* prev;
    curr = prev = l->head;
    while (curr && curr->key != key) {
	prev = curr;
	curr = curr->next;
    }

    // may not have found key
    if (curr == NULL) {
	printf("not found %d\n", key);
	return;
    }
    node_t* tmp = curr;
    prev->next = curr->next;
    free(tmp);
}

// generic list iterator
void list_apply(list_t* l, list_iterator_t func) {
    node_t* curr = l->head;
    while (curr) {
	(*func)(curr->key);
	curr = curr->next;
    }
}



