#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

// goal: to make a "linked list" of elements in C
typedef struct __node_t node_t;
typedef struct __list_t list_t;
typedef void (*list_iterator_t)(int);

// node_t -- holds elements of list
struct __node_t {
    int key;	
    node_t* next;
};

// list_t -- one of these per list
struct __list_t {
    node_t* head;
};

// methods (functions) on list_t
void list_init(list_t* l) {
    // empty list
    l->head = NULL;
}

// insert method: put something into FRONT of list
void list_insert(list_t* l, int key) {
    node_t* n = malloc(sizeof(node_t));
    assert(n != NULL);
    n->key = key;
    n->next = l->head;
    l->head = n;
}

// insert method #2: insert into BACK of list
void list_insert_back(list_t* l, int key) {
    node_t* n = malloc(sizeof(node_t));
    assert(n != NULL);
    n->key = key;
    n->next = NULL;

    // special case: list is empty
    if (l->head == NULL) {
	l->head = n;
	return;
    }

    // general case: non-empty, have to find end
    node_t* curr;
    node_t* prev;
    curr = prev = l->head;
    while (curr) {
	prev = curr;
	curr = curr->next;
    }
    prev->next = n;
}

void list_delete(list_t* l, int key) {
    if (l->head == NULL) {	
	printf("empty list!\n");
	return;
    }

    if (l->head->key == key) {
	printf("delete from head\n");
	node_t* tmp = l->head;
	l->head = l->head->next;
	free(tmp);
	return;
    }

    node_t* curr;
    node_t* prev;
    curr = prev = l->head;
    while (curr && curr->key != key) {
	prev = curr;
	curr = curr->next;
    }

    // may not have found key
    if (curr == NULL) {
	printf("not found %d\n", key);
	return;
    }
    node_t* tmp = curr;
    prev->next = curr->next;
    free(tmp);
}


// generic list iterator
void list_apply(list_t* l, list_iterator_t func) {
    node_t* curr = l->head;
    while (curr) {
	(*func)(curr->key);
	curr = curr->next;
    }
}




//
// main program
// 

void my_printer(int key) {
    printf("%d\n", key);
}


int main(int argc, char *argv[]) {
    list_t n;
    list_init(&n);

    int i;
    for (i = 1; i < argc; i++) {
	int key = atoi(argv[i]);
	if (key > 0) 
	    list_insert_back(&n, key);
	else
	    list_delete(&n, -key);
    }

    list_apply(&n, my_printer);

    return 0;
}

