#ifndef __zstdio_h__
#define __zstdio_h__

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

FILE *Fopen(const char *filename, const char *mode) {
    FILE *fp = fopen(filename, mode);
    if (fp == NULL) {
	fprintf(stderr, "file %s cannot be opened\n", filename);
	exit(1);
    }
    return fp;
}

void Fwrite(const void *restrict ptr, 
	    size_t size, 
	    size_t nitems, 
	    FILE *restrict stream) {
    size_t rc = fwrite(ptr, size, nitems, stream);
    if (rc != nitems) {
	fprintf(stderr, "fwrite failed\n");
	exit(1);
    }
}

void Fread(void *restrict ptr, 
	   size_t size, 
	   size_t nitems, 
	   FILE *restrict stream) {
    size_t rc = fread(ptr, size, nitems, stream);
    if (rc != nitems) {
	fprintf(stderr, "fread failed\n");
	exit(1);
    }
}

void Fseek(FILE *stream, long offset, int whence) {
    int rc = fseek(stream, offset, whence);
    if (rc != 0) {
	fprintf(stderr, "fseek failed\n");
	exit(1);
    }
}


void Fclose(FILE *stream) {
    fclose(stream);
}

void *Malloc(size_t size) {
    void* rc = malloc(size);
    if (rc == NULL) {
	fprintf(stderr, "malloc failed\n");
	exit(1);
    }
    return rc;
}

#endif // __zstdio_h__
